@@ -24,8 +24,10 @@ NotImplementedError
 
 import os
 import shutil
+import time
 import metator.align as mta
 import metator.io as mio
+import metator.log as mtl
 import metator.network as mtn
 import metator.partition as mtp
 import metator.validation as mtv
@@ -103,7 +105,7 @@ class Network(AbstractCommand):
                                 pipeline.
         -e, --enzyme=STR        The list of restriction enzyme used to digest
                                 the contigs separated by a comma. Example:
-                                DpnII,HinfI.
+                                HpaII,MluCI.
         -n, --normalization=STR If None, do not normalized the count of a
                                 contact by the geometric mean of the coverage of
                                 the contigs. Otherwise it's the type of
@@ -139,6 +141,13 @@ class Network(AbstractCommand):
         if not exists(self.args["--outdir"]):
             os.makedirs(self.args["--outdir"])
 
+        # Enable file logging
+        now = time.strftime("%Y%m%d%H%M%S")
+        log_file = join(
+            self.args["--outdir"], ("metator_network" + now + ".log")
+        )
+        mtl.set_file_handler(log_file)
+
         # Transform integer variables as integer.
         min_qual = int(self.args["--min-quality"])
 
@@ -320,6 +329,13 @@ class Partition(AbstractCommand):
                 )
                 raise ValueError
 
+        # Enable file logging
+        now = time.strftime("%Y%m%d%H%M%S")
+        log_file = join(
+            self.args["--outdir"], ("metator_partition" + now + ".log")
+        )
+        mtl.set_file_handler(log_file)
+
         # Transform numeric variable as numeric
         iterations = int(self.args["--iterations"])
         overlapping_parameter = int(self.args["--overlap"]) / 100
@@ -419,6 +435,7 @@ class Validation(AbstractCommand):
         if not exists(self.args["--outdir"]):
             os.makedirs(self.args["--outdir"])
         recursive_fasta_dir = join(self.args["--outdir"], "recursive_bin")
+        final_fasta_dir = join(self.args["--outdir"], "final_bin")
         if not exists(recursive_fasta_dir):
             os.makedirs(recursive_fasta_dir)
         else:
@@ -432,6 +449,26 @@ class Validation(AbstractCommand):
                     )
                 )
                 raise ValueError
+        if not exists(final_fasta_dir):
+            os.makedirs(final_fasta_dir)
+        else:
+            if self.args["--force"]:
+                shutil.rmtree(final_fasta_dir)
+                os.makedirs(final_fasta_dir)
+            else:
+                logger.error(
+                    "{0} already existed. Remove directory or use -F argument to overwrite it.".format(
+                        final_fasta_dir
+                    )
+                )
+                raise ValueError
+
+        # Enable file logging
+        now = time.strftime("%Y%m%d%H%M%S")
+        log_file = join(
+            self.args["--outdir"], ("metator_validation" + now + ".log")
+        )
+        mtl.set_file_handler(log_file)
 
         # Transform numeric variable as numeric
         iterations = int(self.args["--iterations"])
@@ -456,6 +493,7 @@ class Validation(AbstractCommand):
             self.args["--algorithm"],
             self.args["--assembly"],
             self.args["--contigs"],
+            final_fasta_dir,
             self.args["--fasta"],
             iterations,
             self.args["--network"],
@@ -515,7 +553,7 @@ class Pipeline(AbstractCommand):
                                 pipeline.
         -e, --enzyme=STR        The list of restriction enzyme used to digest
                                 the contigs separated by a comma. Example:
-                                DpnII,HinfI.
+                                HpaII,MluCI.
         -F, --force             If enable, would remove directory of overlapping
                                 bins in the output directory.
         -i, --iterations=INT    Number of iterations of Louvain for the
@@ -546,8 +584,8 @@ class Pipeline(AbstractCommand):
                                 algorithm. [Default: 1.0]
         -s, --size=INT          Threshold size to keep bins in base pair.
                                 [Default: 500000]
-        -S, --start=STR         Start stage of the pipeline. Either fastq or
-                                bam. [Default: fastq]
+        -S, --start=STR         Start stage of the pipeline. Either fastq, bam
+                                or network. [Default: fastq]
         -t, --threads=INT       Number of parallel threads allocated for the
                                 alignement. [Default: 1]
         -T, --tempdir=DIR       Temporary directory. Default to current
@@ -584,6 +622,11 @@ class Pipeline(AbstractCommand):
                 )
                 raise ValueError
 
+        # Enable file logging
+        now = time.strftime("%Y%m%d%H%M%S")
+        log_file = join(self.args["--outdir"], ("metator" + now + ".log"))
+        mtl.set_file_handler(log_file)
+
         # Define variable
         min_qual = int(self.args["--min-quality"])
         iterations = int(self.args["--iterations"])
@@ -648,6 +691,20 @@ class Pipeline(AbstractCommand):
                         )
                     )
                     raise ValueError
+            final_fasta_dir = join(self.args["--outdir"], "final_bin")
+            if not exists(final_fasta_dir):
+                os.makedirs(final_fasta_dir)
+            else:
+                if self.args["--force"]:
+                    shutil.rmtree(final_fasta_dir)
+                    os.makedirs(final_fasta_dir)
+                else:
+                    logger.error(
+                        "{0} already existed. Remove directory or use -F argument to overwrite it.".format(
+                            final_fasta_dir
+                        )
+                    )
+                    raise ValueError
 
             # Check checkM availability
             if not mio.check_checkm():
@@ -758,12 +815,20 @@ class Pipeline(AbstractCommand):
             threads,
         )
 
+        # remove contig_data_network if not an input
+        if start <= 2:
+            contig_data_network_file = join(
+                self.args["--outdir"], "contig_data_network.txt"
+            )
+            os.remove(contig_data_network_file)
+
         # Launch validation if desired.
         if not self.args["--skip-validation"]:
             mtv.recursive_decontamination(
                 self.args["--algorithm"],
                 fasta,
                 contigs_data_file,
+                final_fasta_dir,
                 overlapping_fasta_dir,
                 recursive_iterations,
                 network_file,
@@ -776,6 +841,12 @@ class Pipeline(AbstractCommand):
                 threads,
             )
 
+            # Remove contig_data_partition file
+            contig_data_partition_file = join(
+                self.args["--outdir"], "contig_data_partition.txt"
+            )
+            os.remove(contig_data_partition_file)
+
         # Delete the temporary folder.
         if not self.args["--no-clean-up"]:
             shutil.rmtree(temp_directory)
