@@ -660,9 +660,9 @@ def update_contigs_data(contig_data_file, core_bins, overlapping_bins, outdir):
         core_bin_contigs_number = len(core_bin)
         core_bin_length = sum(core_bin_data.Size)
         # Write the new information
-        contigs_data.iloc[core_bin, 7] = n
-        contigs_data.iloc[core_bin, 8] = core_bin_contigs_number
-        contigs_data.iloc[core_bin, 9] = core_bin_length
+        contigs_data.Core_bin_ID[core_bin] = n
+        contigs_data.Core_bin_contigs[core_bin] = core_bin_contigs_number
+        contigs_data.Core_bin_size[core_bin] = core_bin_length
         n += 1
 
     # Add overlapping information
@@ -673,9 +673,13 @@ def update_contigs_data(contig_data_file, core_bins, overlapping_bins, outdir):
         overlapping_bin_contigs_number = len(overlapping_bin)
         overlapping_bin_length = sum(overlapping_bin_data.Size)
         # Write the new information
-        contigs_data.iloc[overlapping_bin, 10] = i
-        contigs_data.iloc[overlapping_bin, 11] = overlapping_bin_contigs_number
-        contigs_data.iloc[overlapping_bin, 12] = overlapping_bin_length
+        contigs_data.Overlapping_bin_ID[overlapping_bin] = i
+        contigs_data.Overlapping_bin_contigs[
+            overlapping_bin
+        ] = overlapping_bin_contigs_number
+        contigs_data.Overlapping_bin_size[
+            overlapping_bin
+        ] = overlapping_bin_length
 
     # Write the new file
     contig_data_file_2 = join(outdir, "contig_data_partition.txt")
