@@ -15,8 +15,9 @@ bin.
 Functions in this module:
     - checkM
     - compare_bins
-    - louvain_recursif
+    - get_bin_coverage
     - give_results_info
+    - louvain_recursif
     - recursive_decontamination
     - update_recursif_louvain
     - write_bin_contigs
@@ -28,6 +29,7 @@ import metator.partition as mtp
 import networkx as nx
 import os
 import pandas as pd
+import shutil
 import subprocess as sp
 from metator.log import logger
 from os.path import join
@@ -186,6 +188,114 @@ def compare_bins(
     return checkm_summary
 
 
+def get_bin_coverage(bin_summary, contigs_data):
+    """Function to compute the coverage of each bin.
+
+    Parameters:
+    -----------
+    bin_summary : dict
+        Dictionnary with the informations of the final bins kept by MetaTOR.
+    contigs_data : pandas.core.frame.DataFrame
+        Dataframe with the contigs informations.
+
+    Returns:
+    --------
+    dict:
+        Dictionnary with the informations of the final bins kept by MetaTOR with
+        the coverage.
+    """
+    # If no depth files were given put some zero on all columns
+    if contigs_data.Shotgun_coverage[0] == "-":
+        return bin_summary
+    # Else read all contigs data file and compute mean coverage for each bin.
+    for i in range(len(contigs_data)):
+        bin_name = contigs_data.Final_bin[i]
+        if bin_name != "ND":
+            try:
+                bin_summary[bin_name]["Coverage"] += (
+                    contigs_data.Size[i]
+                    * contigs_data.Shotgun_coverage[i]
+                    / int(bin_summary[bin_name]["size"])
+                )
+            except KeyError:
+                bin_summary[bin_name]["Coverage"] = (
+                    contigs_data.Size[i]
+                    * contigs_data.Shotgun_coverage[i]
+                    / int(bin_summary[bin_name]["size"])
+                )
+    return bin_summary
+
+
+def give_results_info(bin_summary):
+    """Function to return the general information about the binning results.
+
+    Parameters:
+    -----------
+    bin_summary : dict
+        Dictionnary with the summary results of the kept bins.
+    """
+
+    # Defined categories of the bins
+    HQ = 0  # Completness >= 90 and Contamination <= 5
+    total_size_HQ = 0
+    MQ = 0  # Completness >= 70 and Contamination <= 10
+    total_size_MQ = 0
+    LQ = 0  # Completness >= 50 and Contamination <= 10
+    total_size_LQ = 0
+    conta_bins = 0  # Completness >= 50 and Contamination > 10
+    total_size_conta_bins = 0
+    others = 0  # Not determined bins.
+    total_size_others = 0
+
+    # Class each bin in a category
+    for bin_name in bin_summary:
+        completness = float(bin_summary[bin_name]["completness"])
+        contamination = float(bin_summary[bin_name]["contamination"])
+        size = int(bin_summary[bin_name]["size"])
+        if completness >= 50:
+            if contamination > 10:
+                conta_bins += 1
+                total_size_conta_bins += size
+            else:
+                if completness >= 90 and contamination <= 5:
+                    HQ += 1
+                    total_size_HQ += size
+                elif completness >= 70:
+                    MQ += 1
+                    total_size_MQ += size
+                else:
+                    LQ += 1
+                    total_size_LQ += size
+        else:
+            others += 1
+            total_size_others += size
+    total = HQ + MQ + LQ + conta_bins + others
+    total_size = (
+        total_size_HQ
+        + total_size_MQ
+        + total_size_LQ
+        + total_size_conta_bins
+        + total_size_others
+    )
+
+    # Return info in the logger:
+    logger.info(
+        "{0} bins have been kept after the recursive iterations.".format(total)
+    )
+    logger.info("Total size of the extracted bins: {0}".format(total_size))
+    logger.info("HQ MAGs: {0}\tTotal Size: {1}".format(HQ, total_size_HQ))
+    logger.info("MQ MAGs: {0}\tTotal Size: {1}".format(MQ, total_size_MQ))
+    logger.info("LQ MAGs: {0}\tTotal Size: {1}".format(LQ, total_size_LQ))
+    logger.info(
+        "Contaminated potential MAGs: {0}\tTotal Size: {1}".format(
+            conta_bins, total_size_conta_bins
+        )
+    )
+    logger.info(
+        "Others bins: {0}\tTotal Size: {1}".format(others, total_size_others)
+    )
+
+
 def louvain_recursif(
     assembly,
     iterations,
@@ -258,7 +368,8 @@ def louvain_recursif(
     # Add new coulumns for recursive information.
     contigs_data["Recursive_bin_ID"] = "0"
     contigs_data["Recursive_bin_contigs"] = "-"
-    contigs_data["Recursive_bin_length"] = "-"
+    contigs_data["Recursive_bin_size"] = "-"
+    contigs_data["Final_bin"] = "ND"
 
     # Default no contamination
     contamination = False
@@ -289,7 +400,7 @@ def louvain_recursif(
             )
 
             # Stop to report info log
-            # logger.setLevel(logging.WARNING)
+            logger.setLevel(logging.WARNING)
 
             # Use Louvain algorithmon the subnetwork.
             if algorithm == "leiden":
@@ -348,66 +459,14 @@ def louvain_recursif(
             # Put back the info log
             logger.setLevel(logging.INFO)
 
-    # Write the new file
-    contig_data_file_2 = join(outdir, "contig_data_recursif.txt")
-    contigs_data.to_csv(contig_data_file_2, sep="\t", header=True, index=False)
-
     return contamination, contigs_data
 
 
-def give_results_info(bin_summary):
-    """Function to return the general information about the binning results.
-
-    Parameters:
-    -----------
-    bin_summary : dict
-        Dictionnary with the summary results of the kept bins.
-    """
-
-    # Defined categories of the bins
-    HQ = 0  # Completness >= 90 and Contamination <= 5
-    MQ = 0  # Completness >= 70 and Contamination <= 10
-    LQ = 0  # Completness >= 50 and Contamination <= 10
-    conta_bins = 0  # Completness >= 50 and Contamination > 10
-    others = 0
-    total_size = 0  # Total size of the extracted bins.
-
-    # Class each bin in a category
-    for bin_name in bin_summary:
-        completness = float(bin_summary[bin_name]["completness"])
-        contamination = float(bin_summary[bin_name]["contamination"])
-        size = int(bin_summary[bin_name]["size"])
-        if completness >= 50:
-            if contamination > 10:
-                conta_bins += 1
-            else:
-                if completness >= 90 and contamination <= 5:
-                    HQ += 1
-                elif completness >= 70:
-                    MQ += 1
-                else:
-                    LQ += 1
-        else:
-            others += 1
-        total_size += size
-    total = HQ + MQ + LQ + conta_bins + others
-
-    # Return info in the logger:
-    logger.info(
-        "{0} bins have been kept after the recursive iterations.".format(total)
-    )
-    logger.info("Total size of the extracted bins: {0}".format(total_size))
-    logger.info("HQ MAGs: {0}".format(HQ))
-    logger.info("MQ MAGs: {0}".format(MQ))
-    logger.info("LQ MAGs: {0}".format(LQ))
-    logger.info("Contaminated potential MAGs: {0}".format(conta_bins))
-    logger.info("Others bins: {0}".format(others))
-
-
 def recursive_decontamination(
     algorithm,
     assembly,
     contig_data_file,
+    final_fasta_dir,
     input_fasta_dir,
     iterations,
     network_file,
@@ -431,6 +490,8 @@ def recursive_decontamination(
         Path to the assembly file used for the partition.
     contig_data_file : str
         Path to the contig data table to update.
+    final_fasta_dir : str
+        Path to write the final fasta decontaminated bins.
     input_fasta_dir : str
         Path to the directory where the fasta bin from the partition are.
     iterations : int
@@ -520,16 +581,32 @@ def recursive_decontamination(
             overlapping_checkm_file, overlapping_taxonomy_file
         )
 
-    # Retrurn some values of efficiency of the binning.
+    # Create fasta directory and copy final bins.
+    for bin_name in bin_summary:
+        dst = join(final_fasta_dir, bin_name + ".fa")
+        if bin_name.split("_")[2] == "0":
+            src = join(input_fasta_dir, bin_name + ".fa")
+        else:
+            src = join(recursive_fasta_dir, bin_name + ".fa")
+        shutil.copyfile(src, dst)
+
+    # Return some values of efficiency of the binning.
     give_results_info(bin_summary)
 
+    # Write relevant bins/contigs information for anvio.
+    binning_file = join(outdir, "binning.txt")
+    contigs_data = write_bins_contigs(bin_summary, contigs_data, binning_file)
+
+    # Compute the abundance of the mags.
+    bin_summary = get_bin_coverage(bin_summary, contigs_data)
+
     # Save bin information in final file
     bin_summary_file = join(outdir, "bin_summary.txt")
     mio.write_checkm_summary(bin_summary, bin_summary_file)
 
-    # Write relevant bins/contigs information for anvio.
-    binning_file = join(outdir, "binning.txt")
-    write_bins_contigs(bin_summary, contigs_data, binning_file)
+    # Write the new file
+    contig_data_file_2 = join(outdir, "contig_data_final.txt")
+    contigs_data.to_csv(contig_data_file_2, sep="\t", header=True, index=False)
 
 
 def update_contigs_data_recursif(
@@ -578,9 +655,11 @@ def update_contigs_data_recursif(
 
         if recursif_bin_contigs_number > 1:
             # Write the new information
-            contigs_data.iloc[recursif_bin, 13] = rec_id
-            contigs_data.iloc[recursif_bin, 14] = recursif_bin_contigs_number
-            contigs_data.iloc[recursif_bin, 15] = recursif_bin_length
+            contigs_data.Recursive_bin_ID[recursif_bin] = rec_id
+            contigs_data.Recursive_bin_contigs[
+                recursif_bin
+            ] = recursif_bin_contigs_number
+            contigs_data.Recursive_bin_size[recursif_bin] = recursif_bin_length
 
             if recursif_bin_length > size:
 
@@ -588,13 +667,13 @@ def update_contigs_data_recursif(
                 contamination = True
 
                 # Defined name of the recursif bin
-                oc_id = contigs_data.iloc[recursif_bin[0], 10]
+                oc_id = contigs_data.Overlapping_bin_ID[recursif_bin[0]]
                 output_file = join(
                     outdir, "MetaTOR_{0}_{1}.fa".format(oc_id, rec_id)
                 )
 
                 # Retrieve names of the contigs
-                list_contigs = list(contigs_data.iloc[recursif_bin, 1])
+                list_contigs = list(contigs_data.Name[recursif_bin])
 
                 # Generate the fasta
                 contigs_file = join(
@@ -625,6 +704,11 @@ def write_bins_contigs(bin_summary, contigs_data, outfile):
         Dataframe with the contigs informations.
     outfile : str
         Path where to write the output file.
+
+    Returns:
+    --------
+    pandas.core.frame.DataFrame
+        Dataframe with the contigs informations with the final bin information.
     """
 
     # Create a list with the id of the bins
@@ -640,39 +724,28 @@ def write_bins_contigs(bin_summary, contigs_data, outfile):
     # Write the contigs id with their bins id in table file
     with open(outfile, "w") as f:
         for i in range(len(contigs_data)):
-            over_id = str(
-                contigs_data.iloc[
-                    i,
-                ]["Overlapping_bin_ID"]
-            )
-            rec_id = str(
-                contigs_data.iloc[
-                    i,
-                ]["Recursive_bin_ID"]
-            )
+            over_id = str(contigs_data.Overlapping_bin_ID[i])
+            rec_id = str(contigs_data.Recursive_bin_ID[i])
             try:
                 rec_ids = list_bin_id[over_id]
+                binned = False
+                # Case of a recursive bin
                 if rec_id in rec_ids:
-                    f.write(
-                        "{0}\tMetaTOR_{1}_{2}\n".format(
-                            contigs_data.iloc[
-                                i,
-                            ]["Name"],
-                            over_id,
-                            rec_id,
-                        )
-                    )
-                # Case where the recursif bins where not kept.
+                    binned = True
+                # Case where the recursive bins where not kept.
                 elif rec_ids == ["0"]:
                     rec_id = "0"
+                    binned = True
+
+                if binned:
+                    final_bin = "MetaTOR_{0}_{1}".format(
+                        over_id,
+                        rec_id,
+                    )
+                    contigs_data.Final_bin[i] = final_bin
                     f.write(
-                        "{0}\tMetaTOR_{1}_{2}\n".format(
-                            contigs_data.iloc[
-                                i,
-                            ]["Name"],
-                            over_id,
-                            rec_id,
-                        )
+                        "{0}\t{1}\n".format(contigs_data.Name[i], final_bin)
                     )
             except KeyError:
                 pass
+    return contigs_data
